
/* 
* ATMEGA644V
* Quarz 11059200 MHZ
* printf("xxx") eingebunden
*/

#include <stdint.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <string.h>

#include <util/delay.h>

#include "i2cmaster.h"


int uart_putchar(char c, FILE *stream);
void ds1307_schreiben(void);
void ds1307_lesen(void);
char rtc_bcd_to_hex(char v);
char rtc_hex_to_bcd(char v);
void ausgabe(void);

#define SLAVE_ADRESSE 0xD0



typedef struct { 			//  -------- Struktur ----------- für globale Daten
	uint8_t control_0;		//C0 		0x00
	uint8_t sekunde;		//sek 		0x08
	uint8_t minute;			//min
	uint8_t stunde;			//std
	uint8_t monat;			//monat
	uint8_t tag	;			//tag
	uint8_t w_tag;			//w_tag
	uint8_t jahr;			//jahr		0x0E
	} MY_BUFFER;

	static MY_BUFFER rtc_buf;


int main(void)
{


	stdout = &mystdout; // bindet printf() ein
	DDRC = 0x03; //SCL/SDA Out
	i2c_init();
	sei();
	uart_init( UART_BAUD_SELECT(115200UL,11059200UL) );

	ds1307_schreiben(); // nur einmalig  zur Eingabe der Uhrzeit starten

while (1)
{


	ds1307_lesen();
	_delay_ms(250);
	_delay_ms(250);
	ausgabe();

}


		return (0);
}

void ausgabe(void) 	// RTC Uhr DS1307 auslesen und auf Display ausgeben,

	{

	printf("Datum: %02d.%02d.%04d   ",rtc_buf.tag,rtc_buf.monat,rtc_buf.jahr+2000); 	// Ausgabe Tag,Monat,Jahr

 	printf("Zeit:  %02d:%02d:%02d\r\n",rtc_buf.stunde,rtc_buf.minute,rtc_buf.sekunde);// Ausgabe Stunde, Minute, Sekunde

	}


void ds1307_schreiben(void)
{



  		i2c_start_wait(SLAVE_ADRESSE+I2C_WRITE);       // set device address and write mode


		i2c_write(0x00); // Startadresse
		_delay_ms(1);
		i2c_write(0x00); // Sekunden
		_delay_ms(1);
		i2c_write(0x24); // Minuten
		_delay_ms(1);
		i2c_write(0x20); // Stunden
		_delay_ms(1);
		i2c_write(0x03); // Tag  in der Woche
		_delay_ms(1);
		i2c_write(0x09); // Tag
		_delay_ms(1);
		i2c_write(0x08); // Monat
		_delay_ms(1);
		i2c_write(0x11); // Jahr
		_delay_ms(1);
		i2c_write(0x90); // Jahr
		_delay_ms(1);

		i2c_stop();      // Bus stop




}

//--------------------------------lesen--------------------------
void ds1307_lesen(void)
{

	i2c_start_wait(SLAVE_ADRESSE+I2C_WRITE);       // set device address and write mode
 	i2c_write(0x00);  // Startadresse
	i2c_stop();
		                           // Bus stop

    i2c_rep_start(SLAVE_ADRESSE+I2C_READ);       // set device address and write mode


	rtc_buf.sekunde  = rtc_bcd_to_hex(i2c_readAck()); 	//sek
	rtc_buf.minute   = rtc_bcd_to_hex(i2c_readAck()); 	//min
	rtc_buf.stunde   = rtc_bcd_to_hex(i2c_readAck()); 	//std
	rtc_buf.w_tag    = rtc_bcd_to_hex(i2c_readAck()); 	//tag in der Woche
	rtc_buf.tag      = rtc_bcd_to_hex(i2c_readAck()); 	//tag
	rtc_buf.monat    = rtc_bcd_to_hex(i2c_readAck()); 	//monat
	rtc_buf.jahr     = rtc_bcd_to_hex(i2c_readNak()); 	//jahr



return;
}




int uart_putchar(char c, FILE *stream) // // bindet printf() ein
{
 	uart_putc(c); //UART
    return 0;
}




char rtc_bcd_to_hex(char v)// rtc_bcd_to_hex: wandelt BCD zu HEX Zahl um
{
  char n;

  n=v;
  n=n & 0xF0;
  n=n/16;
  v =v & 0x0F;
  n=n*10+v;
  return(n);
}

char rtc_hex_to_bcd(char v)// rtc_hex_to_bcd: wandelt HEX zu BCD Zahl um
{
  char n;

  n=v;
  n=n/10;
  n=n*16;
  v =v % 10;
  n=n+v;
  return(n);
}


