#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdint.h>
#include "global.h"
#include <avr/io.h>


#define F_CAL 12800000L         // gemessener Kalibriertakt in 10^-5 Hz
                                // Endung L ist wichtig!


volatile uint8_t ss = 0;    //Sekunde
uint8_t mm = 32;             //Minute
uint8_t hh = 14;             //Stunde
uint8_t dd = 26;              //Tag
uint8_t doW = 2;            //Tag der Woche
uint8_t m = 3;              //Monat
uint8_t y = 13;              //Jahr

volatile int8_t ssW = 0;    //Sekunde der Weckzeit
volatile int8_t hhW = 0;    //Stunde der Weckzeit
volatile int8_t mmW = 0;    //Minute der Weckzeit

uint8_t DCFempfang = 0;
uint8_t Alarmzeit = 0;
uint8_t AlarmEin = 0;
uint8_t SleepEin = 0;

int8_t Modus = 0b00000000;

uint8_t switchHelp = 0;


// EEPROM Daten
int16_t ee_rtc_cal EEMEM = (F_CAL-12800000)*256/100;        // Kalibrierung für RTC, Frequenzfehler in 1/1000 Hz

// beidseitiger Zugriff durch ISR und Hauptprogramm
//volatile uint32_t time;             // 24h Zeitstempel, 1s Auflösung
volatile int16_t rtc_cal;           // Kalibrierung des 32K Quarzes
volatile uint8_t flag_1s;           // Flag für 1s Intervall


int main (void)
{
    DDRA  &= ~(1<<PA0) | ~(1<<PA1) | ~(1<<PA2);
    PORTA |= 0b00000000;


    DDRD = 0b11111111;
    PORTD |= 0b00000000;

    DDRC &= 0b00000000;
    PORTC |= 0x00;

    MAX_initialisieren();


// Analogcomparator ausschalten
    ACSR = 0x80;

// Timer2 konfigurieren
    TCCR2 |= ((1 << CS21));       // Prescaler 8
    ASSR   = (1<< AS2);             // Timer2 asynchron takten mit Uhrenquarz
    long_delay(1000);               // Einschwingzeit des 32kHz Quarzes
    TCCR2 = (1<<COM21) | (1<<WGM21) | (1<<WGM20); // Fast PWM, non inverted
    TCCR2 = 1;                     // Vorteiler 1 -> 7,8ms Überlaufperiode
    OCR2  = 128;                   // PWM, Tastverhältnis 50%
    while((ASSR & (1<< TCR2UB)));  // Warte auf das Ende des Zugriffs
    TIFR  &= ~(1<<TOV2);           // Interrupts löschen
    TIMSK |= (1<<TOIE2);           // Timer overflow Interrupt freischalten


//Timer 1 konfigurieren (CTC-Sound)
    DDRD |= (1<<PD5);                   //PD5 (OC1A) auf Ausgang Speaker angeschlossen
    TCCR1A |= (1<<COM1A0);              //Clear OC1A on compare match
    TCCR1B |= (1<<WGM12) | (1<<CS11);   //CTC WGM Mode 4, Prescaler 8
    OCR1A = 0;

// Timer 0
    TCCR0 |= (1<<WGM01) | (1<<CS01) | (1<<CS00);     // CTC, XTAL / 64
    OCR0 = 250;   // 1ms
    TIMSK |= 1<<OCIE0;




// Interrupts freigeben
    sei();

    timeFromDS1307();

    Modus = MOD_Normal;

    uint8_t ssold = 0;
    uint8_t mmold = 0;

    Anzeigen(ss, mm, hh, dd, m);

    while(1)
    {

        tasterAlarmzeit(PINC, PINC4);
        tasterX(PIND, PIND7);
        tasterDCFempfang(PINC, PINC5);
        tasterSleep(PINC, PINC2);
        tasterAlarmEin(PINC, PINC3);


//        if (Modus == MOD_DCF){
//            deimmAllAus();
//            DCFempfang = 1;
//            PORTD |= (1<<PD2);
//            PORTD &= ~((1<<PD0) | (1<<PD1));
//        }
//        else
        if (Modus == MOD_AlarmHour ||
            Modus == MOD_AlarmMin ||
            Modus == 0b00001010 ||
            Modus == 0b00010010     )
        {
            switchfkt();
        }
        else if (ss != ssold)
        {
            switchfkt();
            AnzeigenSS(ss);
            ssold = ss;
        }

        if (AlarmEin && hh == hhW && mm == mmW && Alarmzeit == 0)
        {
            Modus = MOD_AlarmEin;
            DDRD |= (1<<PD5);                   //PD5 (OC1A) auf Ausgang Speaker angeschlossen
            TCCR1A |= (1<<COM1A0);              //Clear OC1A on compare match
            TCCR1B |= (1<<WGM12) | (1<<CS11);   //CTC WGM Mode 4, Prescaler 8

        }
        else{
            TCCR1B &= ~( (1<<CS12) | (1<<CS11) | (1<<CS10) );  //Timer Stopp
            PORTD |= (1<<PD6);
        }

    }
}



// Timer0 compare interrupt FÜR DREHGEBER DCF-MODUL und Tonlänge


ISR(TIMER0_COMP_vect)             // 1ms for manual movement
{

    if (AlarmEin && mm == mmW && hh == hhW)
    {

//       PORTD &= ~(1<<PD6);
//      //-- Tondauer abgelaufen? -------
//        if (++u16Tick < laenge[u8ActTon]) {
//            u8PauseTicker = 0;
//            return;
//        }
//      //-- Pause zwischen den Noten
//        if (++u8PauseTicker < 50){
//            OCR1A = 0;
//            PORTD |= (1<<PD6);
//            return;
//        }
//      //-- TonArrayEnde? neu beginnen ------
//        if (++u8ActTon > 46 ) u8ActTon = 0;
//      //-- nächster Ton----------------
//        u16Tick = 0;
//        OCR1A = toene[u8ActTon];
//
//
//    }


    if (Alarmzeit)
    {
      int8_t new, diff;

      new = 0;
      if( PHASE_A )
        new = 3;
      if( PHASE_B )
        new ^= 1;                   // convert gray to binary
      diff = last - new;                // difference last - new
      if( diff & 1 ){               // bit 0 = value (1)
        last = new;                 // store new as next last
        enc_delta += (diff & 2) - 1;        // bit 1 = direction (+/-)
      }
    }


    if (DCFempfang)
    {
        DCFempfangen();

    }

}

// Timer2 overflow Interrupt FÜR UHRZEIT

ISR(TIMER2_OVF_vect)
{
    static uint8_t ticks;               // Hilfsvariable für Messintervall
    static int16_t time_error;          // RTC Fehlerkompensation

    // Zeitkritische Dinge, welche am Anfang der ISR stehen müssen!

    OCR2=128;                          // Dummy-Write zur Sicherung des Timings
    // von Timer 2 im asynchronen Modus

    // RTC Fehler korrigieren

    if (time_error>999)                 // RTC zu schnell
    {
        TCNT2 = 2;                      // Zähler einen Schritt zurück setzen (2 Takte Verzögerung!)
        time_error -= 1000;
    }
    else if (time_error<-999)           // RTC zu langsam
    {
        TCNT2 = 4;                      // Zähler einen Schritt vor setzen (2 Takte Verzögerung!)
        time_error += 1000;
    }

    // ab hier ist es nicht mehr zeitkritisch

    // Echtzeituhr
    ticks++;                            // 1/128tel Sekunde

//    if (ticks==128)
    if (ticks == 128)
    {
        // Sekundenintervall
        time_error += rtc_cal;          // RTC Fehler akkumulieren

        ss++;
        // 24h Timer
//        time++;
//        if (time==86400) time=0;        // 24h Überlauf

        ticks=0;
        flag_1s =1;   // setzte Flag für 1s Verarbeitung in main Endlosschleife
    }
}


